class PhModules:
    PLAY_HELPERS = 'playhelpers'
    PYCRATE = 'pycrate'
    PYCRATE_VERSION_SUPPORTING_MULTI_LINE_COMMENTS = '0.5.4'
    SEGNO = 'segno'


class PhApps:
    NOTEPAD_PLUS_PLUS = 'notepad++'
    BEYOND_COMPARE = 'beyond compare'
