import os


class PhTestConst:
    SRC_CODE_PATH = 'D:\\Other\\Github_Self\\playhelpers'
    SRC_CODE_PACKAGE_NAME = 'play_helpers'
    SRC_CODE_PH_BITS_FILE_NAME = 'ph_bits.py'
    SRC_CODE_PH_CONSTANTS_FILE_NAME = 'ph_constants.py'
    SRC_CODE_PH_DATA_MASTER_FILE_NAME = 'ph_data_master.py'
    SRC_CODE_PH_BITS_FILE_PATH = os.sep.join([SRC_CODE_PATH, SRC_CODE_PACKAGE_NAME, SRC_CODE_PH_BITS_FILE_NAME])
    SRC_CODE_PH_CONSTANTS_FILE_PATH = os.sep.join(
        [SRC_CODE_PATH, SRC_CODE_PACKAGE_NAME, SRC_CODE_PH_CONSTANTS_FILE_NAME])
    SRC_CODE_PH_DATA_MASTER_FILE_PATH = os.sep.join(
        [SRC_CODE_PATH, SRC_CODE_PACKAGE_NAME, SRC_CODE_PH_DATA_MASTER_FILE_NAME])
    SRC_CODE_TEST_DIR_NAME = 'test'
    SRC_CODE_TEST_DIR_PATH = os.sep.join([SRC_CODE_PATH, SRC_CODE_PACKAGE_NAME, SRC_CODE_TEST_DIR_NAME])

    SRC_CODES_PH_FILES_NAMES = [
        SRC_CODE_PH_BITS_FILE_NAME,
        SRC_CODE_PH_CONSTANTS_FILE_NAME,
        SRC_CODE_PH_DATA_MASTER_FILE_NAME,
    ]

    SRC_CODES_PH_FILES_PATHS = [
        SRC_CODE_PH_BITS_FILE_PATH,
        SRC_CODE_PH_CONSTANTS_FILE_PATH,
        SRC_CODE_PH_DATA_MASTER_FILE_PATH,
    ]
